@@ -112,7 +112,7 @@ let horizontal_line =Text.horizontal_line;;
 let put_separator =Text.put_separator;;
 let unskip = Text.unskip;;
 let put_tag =Text.put_tag;;
-let put_nbsp =Text.put_nbsp;;
+let put_hspace =Text.put_hspace;;
 let put_open_group =Text.put_open_group;;
 let put_close_group =Text.put_close_group;;
 let put_in_math =Text.put_in_math;;
