@@ -27,6 +27,12 @@ val translate_in : char -> (unit -> int) -> unichar
 val translate_out : unichar -> (char -> unit) -> unit
 
 (* Diacritical marks *)
+val put_empty : (unichar -> unit) -> unichar -> unit
+val apply_accent :
+    (char -> unit) -> (unichar -> unit) ->
+      (char -> unichar) -> (char -> unichar) option ->
+        unichar -> char -> unit
+
 val grave : char -> unichar
 val acute : char -> unichar
 val circumflex : char -> unichar
@@ -61,6 +67,18 @@ val html_put : (string -> unit) -> (char -> unit) -> unichar -> unit
 val null : unichar
 val space : unichar
 val nbsp : unichar
+val visible_space : unichar
+val emsp : unichar
+val ensp : unichar
+val emsp13 : unichar
+val emsp14 : unichar
+val six_per_em_space : unichar
+val hairsp : unichar
+val zero_width_space : unichar
+val zero_width_joiner : unichar
+val six_per_em_nbsp : unichar
+val medium_space : unichar
+val word_joiner : unichar
 val acute_alone : unichar
 val grave_alone : unichar
 val circum_alone : unichar
@@ -93,3 +111,17 @@ val tprime : unichar
 val rprime : unichar
 val rdprime : unichar
 val rtprime : unichar
+
+(* Combinations *)
+val comb_cedilla : char -> unichar
+val comb_grave : char -> unichar
+val comb_acute : char -> unichar
+
+(* Double diacritics *)
+val double_inverted_breve : unichar
+
+(* Apply accent on unicode entity *)
+val on_entity :
+    (char -> unit) -> (unichar -> unit) ->
+      (char -> unichar) -> (char -> unichar) option ->
+        unichar -> string -> unit
